fn solve_part1(input: &str) -> i32 {
    let (horiz, _, depth) = solve(input);
    horiz * depth
}

fn solve_part2(input: &str) -> i32 {
    let (horiz, depth, _) = solve(input);
    horiz * depth
}

fn solve(input: &str) -> (i32, i32, i32) {
    let (mut horiz, mut depth, mut aim) = (0, 0, 0);

    for (dir, dist) in parse_input(input) {
        match dir {
            Some("forward") => {
                horiz += dist;
                depth += aim * dist;
            }
            Some("up") => aim -= dist,
            Some("down") => aim += dist,
            other => panic!("{:#?} is invalid", other),
        }
    }

    (horiz, depth, aim)
}

fn parse_input(input: &str) -> Vec<(Option<&str>, i32)> {
    input
        .lines()
        .map(|s| {
            let mut it = s.split_whitespace();

            let dir = it.next();
            let dist = it
                .next()
                .map(|s| s.parse().ok())
                .flatten()
                .expect("parse &str as i32");

            (dir, dist)
        })
        .collect()
}

pub fn run(input: String) {
    println!("--- Part 1 ---");
    println!("Answer: {}", solve_part1(&input));
    println!();
    println!("--- Part 2 ---");
    println!("Answer: {}", solve_part2(&input))
}

#[cfg(test)]
mod tests {
    const PART_1_EXAMPLE_INPUT: &str = "forward 5\ndown 5\nforward 8\nup 3\ndown 8\nforward 2";

    #[test]
    fn part1() {
        assert_eq!(super::solve_part1(PART_1_EXAMPLE_INPUT), 150);
    }

    #[test]
    fn part2() {
        assert_eq!(super::solve_part2(PART_1_EXAMPLE_INPUT), 900);
    }
}
